@@ -3,23 +3,21 @@ from __future__ import annotations
 
 import contextlib
 import os
-import sys
-import urllib.parse as urlparse
 import uuid
+import warnings
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Any, Iterator, Mapping, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, Iterator, Literal, Sequence, Union, cast
 
 import async_retriever as ar
-import cytoolz as tlz
+import cytoolz.curried as tlz
 import pyproj
-from loguru import logger
 from owslib.wfs import WebFeatureService
 from owslib.wms import WebMapService
-from shapely.geometry import LineString, MultiPoint, Point, Polygon
+from yarl import URL
 
-from . import utils
-from .exceptions import (
+from pygeoogc import utils
+from pygeoogc.exceptions import (
     InputTypeError,
     InputValueError,
     MissingInputError,
@@ -28,61 +26,22 @@ from .exceptions import (
     ZeroMatchedError,
 )
 
-logger.configure(
-    handlers=[
-        {
-            "sink": sys.stdout,
-            "colorize": True,
-            "format": " | ".join(
-                [
-                    "{time:YYYY-MM-DD at HH:mm:ss}",  # noqa: FS003
-                    "{name: ^15}.{function: ^15}:{line: >3}",  # noqa: FS003
-                    "{message}",  # noqa: FS003
-                ]
-            ),
-        }
-    ]
-)
-if os.environ.get("HYRIVER_VERBOSE", "false").lower() == "true":
-    logger.enable("pygeoogc")
-else:
-    logger.disable("pygeoogc")
-CRSTYPE = Union[int, str, pyproj.CRS]
+if TYPE_CHECKING:
+    CRSTYPE = Union[int, str, pyproj.CRS]
+
+__all__ = ["ArcGISRESTfulBase", "WMSBase", "WFSBase"]
 
 
-def validate_version(val: str, valid_versions: list[str]) -> str:
-    """Validate version from a list of valid versions.
-
-    Parameters
-    ----------
-    val : str
-        Input version value.
-    valid_versions : list of str
-        List of valid versions.
-
-    Returns
-    -------
-    str
-        Validated version value.
-    """
-    if val not in valid_versions:
-        raise InputValueError("version", valid_versions)
-    return val
-
-
-def split_url(url: str, layer: int | None) -> tuple[str, int]:
+def _extract_layer(url: str, layer: int | None) -> tuple[str, int]:
     """Check if layer is included in url, if so separate and return them."""
-    url = urlparse.unquote(url)
+    url_obj = URL(url[:-1] if url.endswith("/") else url)
     if layer is None:
-        url_split = urlparse.urlsplit(url.rstrip("/"))
-        url_path, lyr = url_split.path.rsplit("/", 1)
-        url_split = url_split._replace(path=url_path)
         try:
-            return urlparse.urlunsplit(url_split), int(lyr)
+            return url_obj.parent.human_repr(), int(url_obj.parts[-1])
         except ValueError as ex:
             msg = "Either layer must be passed as an argument or be included in ``base_url``"
             raise MissingInputError(msg) from ex
-    return url, layer
+    return url_obj.human_repr(), layer
 
 
 class ArcGISRESTfulBase:
@@ -132,13 +91,11 @@ class ArcGISRESTfulBase:
         verbose: bool = False,
         disable_retry: bool = False,
     ) -> None:
-        self.base_url, self.layer = split_url(base_url, layer)
+        self.base_url, self.layer = _extract_layer(base_url, layer)
         self.outformat = outformat
         self.outfields = outfields if isinstance(outfields, (list, tuple)) else [outfields]
         self.crs = utils.validate_crs(crs)
         self.max_workers = max_workers
-        if self.max_workers < 1:
-            raise InputTypeError("max_workers", "positive integer > 1")
         self.verbose = verbose
         self.disable_retry = disable_retry
 
@@ -202,6 +159,13 @@ class ArcGISRESTfulBase:
         self.field_types = {
             f["name"]: f["type"].replace("esriFieldType", "").lower() for f in rjson["fields"]
         }
+        if "alias" in rjson["fields"][0]:
+            self.field_types.update(
+                {
+                    f["alias"]: f["type"].replace("esriFieldType", "").lower()
+                    for f in rjson["fields"]
+                }
+            )
 
         with contextlib.suppress(KeyError):
             self.feature_types = dict(
@@ -230,12 +194,10 @@ class ArcGISRESTfulBase:
             raise ZeroMatchedError
 
         self.n_features = len(oid_list)
-        if not self.disable_retry and self.verbose:
-            logger.info(f"Found {self.n_features:,} feature(s) in the requested region.")
-        return tlz.partition_all(self.max_nrecords, [str(i) for i in oid_list])  # type: ignore
+        return tlz.partition_all(self.max_nrecords, [str(i) for i in oid_list])
 
     def _cleanup_resp(
-        self, resp: list[dict[str, Any]], payloads: Sequence[dict[str, str]]
+        self, resp: list[dict[str, Any]], payloads: list[dict[str, str]]
     ) -> list[dict[str, Any]]:
         """Remove failed responses."""
         fails = [i for i, r in enumerate(resp) if "error" in r]
@@ -258,28 +220,29 @@ class ArcGISRESTfulBase:
                     self.return_m = bool(payloads[0]["ReturnM"])
                     self.return_geom = bool(payloads[0]["returnGeometry"])
                     self.total_n_features = self.n_features
+
                     if len(fails) > 1:
-                        logger.warning(f"Found {len(fails)} failed requests. Retrying ...")
-                    else:
-                        logger.warning("Found 1 failed request. Retrying ...")
+                        msg = f"Found {len(fails)} failed requests. Retrying ..."
+                        warnings.warn(msg, UserWarning, stacklevel=2)
+                    elif len(fails) == 1:
+                        msg = "Found 1 failed request. Retrying ..."
+                        warnings.warn(msg, UserWarning, stacklevel=2)
+
                     resp.extend(self.retry_failed_requests())
 
                     if self.n_missing > 0:
-                        logger.warning(
-                            " ".join(
-                                [
-                                    f"Total of {self.n_missing} out of {self.total_n_features}",
-                                    "requested features are not available in the dataset.",
-                                    "Returning the successfully retrieved features.",
-                                    "The failed object IDs have been saved in the",
-                                    f"file {self.failed_path}. The service returned the",
-                                    "following error message for the failed requests:\n",
-                                    err,
-                                ]
-                            )
+                        msg = " ".join(
+                            [
+                                f"Total of {self.n_missing} out of {self.total_n_features}",
+                                "requested features are not available in the dataset.",
+                                "Returning the successfully retrieved features.",
+                                "The failed object IDs have been saved in the",
+                                f"file {self.failed_path}. The service returned the",
+                                "following error message for the failed requests:\n",
+                                err,
+                            ]
                         )
-                    else:
-                        logger.info("All feature IDs have been successfully retrieved.")
+                        warnings.warn(msg, UserWarning, stacklevel=2)
 
         return resp
 
@@ -327,31 +290,6 @@ class ArcGISRESTfulBase:
             raise ZeroMatchedError
         return resp
 
-    def esri_query(
-        self,
-        geom: (LineString | Polygon | Point | MultiPoint | tuple[float, float, float, float]),
-        geo_crs: CRSTYPE = 4326,
-    ) -> Mapping[str, str]:
-        """Generate geometry queries based on ESRI template."""
-        geom = utils.match_crs(geom, geo_crs, self.crs)
-
-        if isinstance(geom, tuple) and len(geom) == 4:
-            return utils.ESRIGeomQuery(geom, self.out_sr).bbox()
-
-        if isinstance(geom, Point):
-            return utils.ESRIGeomQuery((geom.x, geom.y), self.out_sr).point()
-
-        if isinstance(geom, MultiPoint):
-            return utils.ESRIGeomQuery([(g.x, g.y) for g in geom.geoms], self.out_sr).multipoint()
-
-        if isinstance(geom, Polygon):
-            return utils.ESRIGeomQuery(geom, self.out_sr).polygon()
-
-        if isinstance(geom, LineString):
-            return utils.ESRIGeomQuery(geom, self.out_sr).polyline()
-
-        raise InputTypeError("geom", "LineString, Polygon, Point, MultiPoint, tuple")
-
     def _retry(
         self, return_m: bool, return_geo: bool, partition_fac: float
     ) -> list[dict[str, Any]]:
@@ -378,31 +316,34 @@ class ArcGISRESTfulBase:
         partition_fac = [(0.5 - fac_min) / (n_retry - 1) * i + fac_min for i in range(n_retry)]
         features = []
         for f in partition_fac[::-1]:
-            try:
+            with contextlib.suppress(ServiceError):
                 features.append(self._retry(self.return_m, self.return_geom, f))
-            except ServiceError:
-                continue
 
         features = list(tlz.concat(features))
         self.n_missing -= len(features)
         self.disable_retry = retry
         os.environ["HYRIVER_CACHE_DISABLE"] = f"{caching}".lower()
-        return features  # type: ignore[return-value]
+        return features
 
     def get_response(
-        self, url: str, payloads: Sequence[dict[str, str]], method: str = "GET"
+        self,
+        url: str,
+        payloads: list[dict[str, str]],
+        method: Literal["get", "GET", "post", "POST"] = "GET",
     ) -> list[dict[str, Any]]:
         """Send payload and get the response."""
         req_key = "params" if method == "GET" else "data"
         try:
-            return ar.retrieve_json(
+            resp = ar.retrieve_json(
                 [url] * len(payloads),
                 [{req_key: p} for p in payloads],
                 request_method=method,
                 max_workers=self.max_workers,
             )
+            resp = cast("list[dict[str, Any]]", resp)
         except ValueError:
             raise ZeroMatchedError
+        return resp
 
     def __repr__(self) -> str:
         """Print the service configuration."""
@@ -448,7 +389,7 @@ class WMSBase:
     """
 
     url: str
-    layers: str | list[str] = ""
+    layers: str | int | Sequence[str | int] = ""
     outformat: str = ""
     version: str = "1.3.0"
     crs: CRSTYPE = 4326
@@ -456,8 +397,12 @@ class WMSBase:
 
     def __post_init__(self) -> None:
         """Validate crs."""
-        self.crs_str = utils.validate_crs(self.crs)
-        self.version = validate_version(self.version, ["1.1.1", "1.3.0"])
+        self.layers = (
+            [str(self.layers)] if isinstance(self.layers, (str, int)) else list(self.layers)
+        )
+        self.crs_str = utils.validate_crs(self.crs).lower()
+        if self.version not in ("1.1.1", "1.3.0"):
+            raise InputValueError("version", ("1.1.1", "1.3.0"))
         self.get_service_options()
         if self.validation:
             self.validate_wms()
@@ -478,8 +423,7 @@ class WMSBase:
 
     def validate_wms(self) -> None:
         """Validate input arguments with the WMS service."""
-        layers = [self.layers] if isinstance(self.layers, str) else self.layers
-        if any(lyr not in self.available_layer.keys() for lyr in layers):
+        if any(lyr not in self.available_layer.keys() for lyr in self.layers):  # type: ignore
             raise InputValueError(
                 "layers", (f"{n} for {t}" for n, t in self.available_layer.items())
             )
@@ -487,11 +431,11 @@ class WMSBase:
         if self.outformat not in self.available_outformat:
             raise InputValueError("outformat", self.available_outformat)
 
-        if any(self.crs_str.lower() not in self.available_crs[lyr] for lyr in layers):
+        if any(self.crs_str not in self.available_crs[lyr] for lyr in self.layers):  # type: ignore
             _valid_crss = (
                 f"{lyr}: {', '.join(cs)}\n"
                 for lyr, cs in self.available_crs.items()
-                if lyr in layers
+                if lyr in self.layers
             )
             raise InputValueError("CRS", _valid_crss)
 
@@ -506,12 +450,11 @@ class WMSBase:
 
     def __repr__(self) -> str:
         """Print the services properties."""
-        layers = self.layers if isinstance(self.layers, list) else [self.layers]
         return (
             "Connected to the WMS service with the following properties:\n"
             + f"URL: {self.url}\n"
             + f"Version: {self.version}\n"
-            + f"Layers: {', '.join(lyr for lyr in layers)}\n"
+            + f"Layers: {', '.join(str(lyr) for lyr in self.layers)}\n"  # type: ignore
             + f"Output Format: {self.outformat}\n"
             + f"Output CRS: {self.crs_str}"
         )
@@ -563,16 +506,25 @@ class WFSBase:
     def __post_init__(self) -> None:
         """Validate crs."""
         self.crs_str = utils.validate_crs(self.crs)
-        self.version = validate_version(self.version, ["1.1.0", "2.0.0"])
+
         if self.version == "2.0.0":
             self.nfeat_key = "numberMatched="
             self.count_key = "count"
-        else:
+        elif self.version == "1.1.0":
             self.nfeat_key = "numberOfFeatures="
             self.count_key = "maxFeatures"
-        valid_methods = ["json", "binary", "text"]
-        if self.read_method not in valid_methods:
-            raise InputValueError("read_method", valid_methods)
+        else:
+            raise InputValueError("version", ("1.1.0", "2.0.0"))
+
+        if self.read_method == "text":
+            self.retrieve = ar.retrieve_text
+        elif self.read_method == "bytes":
+            self.retrieve = ar.retrieve_binary
+        elif self.read_method == "json":
+            self.retrieve = ar.retrieve_json
+        else:
+            raise InputValueError("read_method", ("json", "binary", "text"))
+
         self.get_service_options()
         if self.validation:
             self.validate_wfs()
@@ -595,7 +547,19 @@ class WFSBase:
             for lyr in self.available_layer
         }
 
-        self.schema = {lyr: wfs.get_schema(lyr) for lyr in self.available_layer}
+        self.schema = {}
+        if self.layer:
+            if self.layer not in self.available_layer:
+                raise InputValueError("layers", self.available_layer)
+            layers = [self.layer]
+        else:
+            layers = self.available_layer
+
+        for lyr in layers:
+            try:
+                self.schema[lyr] = wfs.get_schema(lyr)
+            except KeyError:
+                self.schema[lyr] = None
 
     def validate_wfs(self) -> None:
         """Validate input arguments with the WFS service."""
@@ -617,22 +581,26 @@ class WFSBase:
     def sort_params(
         self, sort_attr: str | None, nfeatures: int, start_index: int
     ) -> dict[str, str]:
-        """Get the sort parameters for the WFS request."""
+        """Get sort parameters for a WFS request."""
         if nfeatures <= self.max_nrecords:
             return {}
 
-        valid_attrs = self.schema[self.layer]["properties"]
+        schema = self.schema[self.layer]
+        valid_attrs = schema["properties"] if schema else []
         if sort_attr is None:
-            sort_attr = next(
-                (a for a in self.schema[self.layer]["properties"] if "id" in a.lower()), None
-            )
+            if schema is None:
+                msg = "sort_attr is None and no id column found in the schema."
+                msg += " Please set the sort_attr manually."
+                raise MissingInputError(msg)
+
+            sort_attr = next((a for a in valid_attrs if "id" in a.lower()), None)
             if sort_attr is None:
                 msg = "sort_attr is None and no id column found in the schema."
                 msg += " Please set the sort_attr manually. Available columns are:\n"
                 msg += ", ".join(list(valid_attrs))
                 raise MissingInputError(msg)
 
-        if sort_attr not in valid_attrs:
+        if valid_attrs and sort_attr not in valid_attrs:
             raise InputValueError("sort_attr", list(valid_attrs))
         return {
             "startIndex": str(start_index),
