@@ -3,21 +3,24 @@ from __future__ import annotations
 
 import itertools
 import uuid
-from typing import TYPE_CHECKING, Any, Iterator, NamedTuple, Union, cast
+from typing import TYPE_CHECKING, Any, Iterator, NamedTuple, Sequence, Union, cast
 
 import async_retriever as ar
 import pyproj
 import shapely.ops as ops
-from shapely.geometry import LineString, MultiPoint, MultiPolygon, Point, Polygon
+from shapely import LineString, MultiPoint, MultiPolygon, Point, Polygon
 
-from . import utils
-from .core import ArcGISRESTfulBase, WFSBase, WMSBase
-from .exceptions import InputTypeError, InputValueError, ZeroMatchedError
+from pygeoogc import utils
+from pygeoogc.core import ArcGISRESTfulBase, WFSBase, WMSBase
+from pygeoogc.exceptions import InputTypeError, InputValueError, ServiceError, ZeroMatchedError
 
 if TYPE_CHECKING:
     from ssl import SSLContext
 
-CRSTYPE = Union[int, str, pyproj.CRS]
+    RESPONSE = Union[
+        "list[str]", "list[bytes]", "list[dict[str, Any]]", "list[list[dict[str, Any]]]"
+    ]
+    CRSTYPE = Union[int, str, pyproj.CRS]
 
 
 class ArcGISRESTful:
@@ -154,7 +157,7 @@ class ArcGISRESTful:
         elif isinstance(geom, list) and all(len(g) == 2 for g in geom):
             geom = MultiPoint(geom)
 
-        geom_query = self.client.esri_query(geom, geo_crs)
+        geom_query = utils.esri_query(geom, geo_crs, self.client.crs)
 
         payload = {
             **geom_query,
@@ -196,13 +199,9 @@ class ArcGISRESTful:
         if field not in self.client.valid_fields:
             raise InputValueError("field", self.client.valid_fields)
 
-        if not isinstance(ids, (str, int, list, tuple)):
-            raise InputTypeError("ids", "str or list")
+        ids_ls = [ids] if isinstance(ids, (str, int)) else list(ids)
 
-        ids_ls = [str(ids)] if isinstance(ids, (str, int)) else ids
-
-        ftype = self.client.field_types[field]
-        if "string" in ftype:
+        if "string" in self.client.field_types.get(field, ""):
             fids = ", ".join(f"'{i}'" for i in set(ids_ls))
         else:
             fids = ", ".join(f"{i}" for i in set(ids_ls))
@@ -340,10 +339,9 @@ class WMS:
         self.version = self.client.version
         self.crs = self.client.crs
         self.crs_str = self.client.crs_str
+        self.is_geographic = pyproj.CRS(self.crs).is_geographic
         self.ssl = ssl
-        self.layers = (
-            [self.client.layers] if isinstance(self.client.layers, str) else self.client.layers
-        )
+        self.layers = self.client.layers
 
     def get_validlayers(self) -> dict[str, str]:
         """Get the layers supported by the WMS service."""
@@ -415,7 +413,7 @@ class WMS:
             lyr, bnds = args
             _bbox, counter, _width, _height = bnds
 
-            if self.version != "1.1.1" and pyproj.CRS(self.crs_str).is_geographic and not always_xy:
+            if self.version != "1.1.1" and self.is_geographic and not always_xy:
                 _bbox = (_bbox[1], _bbox[0], _bbox[3], _bbox[2])
             _payload = payload.copy()
             _payload["bbox"] = f'{",".join(str(c) for c in _bbox)}'
@@ -424,7 +422,9 @@ class WMS:
             _payload["layers"] = lyr
             return f"{lyr}_dd_{counter}", _payload
 
-        _lyr_payloads = (_get_payloads(i) for i in itertools.product(self.layers, bounds))
+        _lyr_payloads = (
+            _get_payloads(i) for i in itertools.product(self.layers, bounds)  # type: ignore
+        )
         layers, payloads = zip(*_lyr_payloads)
         layers = cast("tuple[str]", layers)
         payloads = cast("tuple[dict[str, str]]", payloads)
@@ -502,7 +502,7 @@ class WFS(WFSBase):
         box_crs: CRSTYPE = 4326,
         always_xy: bool = False,
         sort_attr: str | None = None,
-    ) -> list[str | bytes | dict[str, Any]]:
+    ) -> RESPONSE:
         """Get data from a WFS service within a bounding box.
 
         Parameters
@@ -559,9 +559,7 @@ class WFS(WFSBase):
             for i in range(0, nfeatures, self.max_nrecords)
         ]
 
-        return ar.retrieve(
-            [self.url] * len(payloads), self.read_method, [{"params": p} for p in payloads]
-        )
+        return self.retrieve([self.url] * len(payloads), [{"params": p} for p in payloads])
 
     def getfeature_bygeom(
         self,
@@ -570,7 +568,7 @@ class WFS(WFSBase):
         always_xy: bool = False,
         predicate: str = "INTERSECTS",
         sort_attr: str | None = None,
-    ) -> list[str | bytes | dict[str, Any]]:
+    ) -> RESPONSE:
         """Get features based on a geometry.
 
         Parameters
@@ -638,8 +636,8 @@ class WFS(WFSBase):
     def getfeature_byid(
         self,
         featurename: str,
-        featureids: list[int | str] | int | str,
-    ) -> list[str | bytes | dict[str, Any]]:
+        featureids: Sequence[int | str] | int | str,
+    ) -> RESPONSE:
         """Get features based on feature IDs.
 
         Parameters
@@ -658,9 +656,6 @@ class WFS(WFSBase):
         if featurename not in valid_features:
             raise InputValueError("featurename", list(valid_features))
 
-        if not isinstance(featureids, (str, int, list, tuple)):
-            raise InputTypeError("featureids", "str or list of str")
-
         featureids = [featureids] if isinstance(featureids, (str, int)) else list(featureids)
 
         if not featureids:
@@ -677,7 +672,7 @@ class WFS(WFSBase):
 
     def getfeature_byfilter(
         self, cql_filter: str, method: str = "GET", sort_attr: str | None = None
-    ) -> list[str | bytes | dict[str, Any]]:
+    ) -> RESPONSE:
         """Get features based on a valid CQL filter.
 
         Notes
@@ -724,8 +719,10 @@ class WFS(WFSBase):
         else:
             headers = {"content-type": "application/x-www-form-urlencoded"}
             resp = ar.retrieve_text([self.url], [{"data": payload, "headers": headers}], "POST")
-
-        nfeatures = int(resp[0].split(self.nfeat_key)[-1].split(" ")[0].strip('"'))
+        try:
+            nfeatures = int(resp[0].split(self.nfeat_key)[-1].split(" ")[0].strip('"'))
+        except (IndexError, ValueError) as ex:
+            raise ServiceError(resp[0], self.url) from ex
 
         payloads = [
             {
@@ -740,16 +737,12 @@ class WFS(WFSBase):
             }
             for i in range(0, nfeatures, self.max_nrecords)
         ]
-
         if method == "GET":
-            return ar.retrieve(
-                [self.url] * len(payloads), self.read_method, [{"params": p} for p in payloads]
-            )
+            return self.retrieve([self.url] * len(payloads), [{"params": p} for p in payloads])
 
         headers = {"content-type": "application/x-www-form-urlencoded"}
-        return ar.retrieve(
+        return self.retrieve(
             [self.url] * len(payloads),
-            self.read_method,
             [{"data": p, "headers": headers} for p in payloads],
             "POST",
         )
@@ -783,7 +776,7 @@ class RESTfulURLs(NamedTuple):
     )
     nid: str = "https://nid.sec.usace.army.mil/api"
     airmap: str = "https://api.airmap.com/elevation/v1/ele"
-    nm_pqs: str = "https://nationalmap.gov/epqs/pqs.php"
+    nm_pqs: str = "https://epqs.nationalmap.gov/v1/json"
     pygeoapi: str = "https://labs.waterdata.usgs.gov/api/nldi/pygeoapi/processes"
     nm_3dep_index: str = (
         "https://index.nationalmap.gov/arcgis/rest/services/3DEPElevationIndex/MapServer"
