@@ -1,16 +1,28 @@
 """Some utilities for PyGeoOGC."""
 from __future__ import annotations
 
+import contextlib
 import itertools
 import math
 import os
 import warnings
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Any, Callable, Generator, List, Mapping, Tuple, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Generator,
+    Mapping,
+    Sequence,
+    TypeVar,
+    Union,
+    cast,
+    overload,
+)
 
 import async_retriever as ar
-import cytoolz as tlz
+import cytoolz.curried as tlz
 import defusedxml.ElementTree as ETree
 import joblib
 import pyproj
@@ -22,27 +34,28 @@ from requests.adapters import HTTPAdapter
 from requests.exceptions import RequestException
 from requests_cache import CachedSession, Response
 from requests_cache.backends.sqlite import SQLiteCache
+from shapely import LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon
+from shapely import box as shapely_box
 from shapely import ops
-from shapely.geometry import LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon
-from shapely.geometry import box as shapely_box
 from urllib3.exceptions import InsecureRequestWarning
 
 from pygeoogc import cache_keys
 from pygeoogc.exceptions import InputTypeError, InputValueError, ServiceError
 
-CRSTYPE = Union[int, str, pyproj.CRS]
+if TYPE_CHECKING:
+    CRSTYPE = Union[int, str, pyproj.CRS]
+    GEOM = TypeVar(
+        "GEOM",
+        Point,
+        MultiPoint,
+        Polygon,
+        MultiPolygon,
+        LineString,
+        MultiLineString,
+        "tuple[float, float, float, float]",
+        "list[tuple[float, float]]",
+    )
 BOX_ORD = "(west, south, east, north)"
-G = TypeVar(
-    "G",
-    Point,
-    MultiPoint,
-    Polygon,
-    MultiPolygon,
-    LineString,
-    MultiLineString,
-    Tuple[float, float, float, float],
-    List[Tuple[float, float]],
-)
 MAX_CONN = 10
 CHUNK_SIZE = int(100 * 1024 * 1024)  # 100 MB
 
@@ -61,7 +74,10 @@ def check_response(resp: str) -> str:
         try:
             return str(root[-1][0].text).strip()
         except IndexError:
-            return str(root[-1].text).strip()
+            try:
+                return str(root[-1].text).strip()
+            except IndexError:
+                return str(root.text).strip()
 
 
 class RetrySession:
@@ -213,14 +229,14 @@ def _prepare_requests_args(
     urls: list[str] | str,
     kwds: list[dict[str, dict[Any, Any]]] | dict[str, dict[Any, Any]] | None,
     method: str,
-    fnames: str | Path | list[str | Path] | None,
+    fnames: str | Path | Sequence[str | Path] | None,
     file_prefix: str,
     file_extention: str,
 ) -> tuple[
     tuple[str, ...], tuple[dict[str, None | dict[Any, Any]], ...], Generator[Path, None, None]
 ]:
     """Get url and kwds for streaming download."""
-    url_list = tuple(urls) if isinstance(urls, (list, tuple)) else (urls,)
+    url_list = (urls,) if isinstance(urls, str) else tuple(urls)
 
     if kwds is None:
         if method == "GET":
@@ -228,7 +244,7 @@ def _prepare_requests_args(
         else:
             kwd_list = ({"data": None},) * len(url_list)
     else:
-        kwd_list = tuple(kwds) if isinstance(kwds, (list, tuple)) else (kwds,)  # type: ignore
+        kwd_list = (kwds,) if isinstance(kwds, dict) else tuple(kwds)
     key_list = {k for keys in kwd_list for k in keys}
     valid_keys = ("params", "data", "json", "headers")
     if any(k not in valid_keys for k in key_list):
@@ -246,12 +262,13 @@ def _prepare_requests_args(
             for u, p in zip(url_list, kwd_list)
         )
     else:
-        f_list = tuple(fnames) if isinstance(fnames, (list, tuple)) else (fnames,)
+        f_list = (fnames,) if isinstance(fnames, (str, Path)) else tuple(fnames)
         if len(url_list) != len(f_list):
             raise InputTypeError("urls/fnames", "lists of same length")
         files = (Path(f) for f in f_list)
-
-    return url_list, kwd_list, files  # type: ignore[return-value]
+    url_list = cast("tuple[str, ...]", url_list)
+    kwd_list = cast("tuple[dict[str, None | dict[Any, Any]], ...]", kwd_list)
+    return url_list, kwd_list, files
 
 
 def _download(
@@ -271,10 +288,40 @@ def _download(
     return fname
 
 
+@overload
+def streaming_download(
+    urls: str,
+    kwds: dict[str, dict[Any, Any]] | None = None,
+    fnames: str | Path | None = None,
+    file_prefix: str = "",
+    file_extention: str = "",
+    method: str = "GET",
+    ssl: bool = True,
+    chunk_size: int = CHUNK_SIZE,
+    n_jobs: int = MAX_CONN,
+) -> Path:
+    ...
+
+
+@overload
+def streaming_download(
+    urls: list[str],
+    kwds: list[dict[str, dict[Any, Any]]] | None = None,
+    fnames: Sequence[str | Path] | None = None,
+    file_prefix: str = "",
+    file_extention: str = "",
+    method: str = "GET",
+    ssl: bool = True,
+    chunk_size: int = CHUNK_SIZE,
+    n_jobs: int = MAX_CONN,
+) -> list[Path]:
+    ...
+
+
 def streaming_download(
     urls: list[str] | str,
     kwds: list[dict[str, dict[Any, Any]]] | dict[str, dict[Any, Any]] | None = None,
-    fnames: str | Path | list[str | Path] | None = None,
+    fnames: str | Path | Sequence[str | Path] | None = None,
     file_prefix: str = "",
     file_extention: str = "",
     method: str = "GET",
@@ -286,7 +333,7 @@ def streaming_download(
 
     Notes
     -----
-    This function uses ``joblib`` with ``loky`` backend.
+    This function runs asynchronously in parallel using ``n_jobs`` threads.
 
     Parameters
     ----------
@@ -340,11 +387,11 @@ def streaming_download(
         func = tlz.partial(session.post, stream=True)
 
     n_jobs = min(n_jobs, len(url_list))
-    fpaths: list[Path] = joblib.Parallel(n_jobs=n_jobs)(
+    fpaths = joblib.Parallel(n_jobs=n_jobs, prefer="threads")(
         joblib.delayed(_download)(func, u, k, f, chunk_size)
         for u, k, f in zip(url_list, kwd_list, files)
     )
-    if n_jobs == 1:
+    if isinstance(urls, str):
         return fpaths[0]
     return fpaths
 
@@ -473,7 +520,7 @@ class ESRIGeomQuery:
         """Query for a point."""
         if isinstance(self.geometry, tuple) and len(self.geometry) == 2:
             geo_type = "esriGeometryPoint"
-            geo_json = dict(zip(("x", "y"), self.geometry))  # type: ignore
+            geo_json = dict(zip(("x", "y"), self.geometry))
             return self._get_payload(geo_type, geo_json)
 
         raise InputTypeError("geometry", "tuple", "(x, y)")
@@ -482,7 +529,7 @@ class ESRIGeomQuery:
         """Query for a multi-point."""
         if isinstance(self.geometry, list) and all(len(g) == 2 for g in self.geometry):
             geo_type = "esriGeometryMultipoint"
-            geo_json = {"points": [[x, y] for x, y in self.geometry]}  # type: ignore
+            geo_json = {"points": [[x, y] for x, y in self.geometry]}
             return self._get_payload(geo_type, geo_json)
 
         raise InputTypeError("geometry", "list of tuples", "[(x, y), ...]")
@@ -491,7 +538,7 @@ class ESRIGeomQuery:
         """Query for a bbox."""
         if isinstance(self.geometry, (tuple, list)) and len(self.geometry) == 4:
             geo_type = "esriGeometryEnvelope"
-            geo_json = dict(zip(("xmin", "ymin", "xmax", "ymax"), self.geometry))  # type: ignore
+            geo_json = dict(zip(("xmin", "ymin", "xmax", "ymax"), self.geometry))
             return self._get_payload(geo_type, geo_json)
 
         raise InputTypeError("geometry", "tuple", BOX_ORD)
@@ -521,7 +568,7 @@ class ESRIGeomQuery:
         raise InputTypeError("geometry", "LineString")
 
 
-def match_crs(geom: G, in_crs: CRSTYPE, out_crs: CRSTYPE) -> G:
+def match_crs(geom: GEOM, in_crs: CRSTYPE, out_crs: CRSTYPE) -> GEOM:
     """Reproject a geometry to another CRS.
 
     Parameters
@@ -543,15 +590,15 @@ def match_crs(geom: G, in_crs: CRSTYPE, out_crs: CRSTYPE) -> G:
     Examples
     --------
     >>> from pygeoogc.utils import match_crs
-    >>> from shapely.geometry import Point
+    >>> from shapely import Point
     >>> point = Point(-7766049.665, 5691929.739)
-    >>> match_crs(point, "epsg:3857", "epsg:4326").xy
+    >>> match_crs(point, "epsg:3857", 4326).xy
     (array('d', [-69.7636111130079]), array('d', [45.44549114818127]))
     >>> bbox = (-7766049.665, 5691929.739, -7763049.665, 5696929.739)
-    >>> match_crs(bbox, "epsg:3857", "epsg:4326")
+    >>> match_crs(bbox, "epsg:3857", 4326)
     (-69.7636111130079, 45.44549114818127, -69.73666165448431, 45.47699468552394)
     >>> coords = [(-7766049.665, 5691929.739)]
-    >>> match_crs(coords, "epsg:3857", "epsg:4326")
+    >>> match_crs(coords, "epsg:3857", 4326)
     [(-69.7636111130079, 45.44549114818127)]
     """
     if pyproj.CRS(in_crs) == pyproj.CRS(out_crs):
@@ -572,12 +619,13 @@ def match_crs(geom: G, in_crs: CRSTYPE, out_crs: CRSTYPE) -> G:
     ):
         return ops.transform(project, geom)
 
-    if isinstance(geom, tuple) and len(geom) == 4:
-        return ops.transform(project, shapely_box(*geom)).bounds
+    if len(geom) == 4:
+        with contextlib.suppress(TypeError):
+            return ops.transform(project, shapely_box(*geom)).bounds
 
-    if isinstance(geom, list) and all(len(c) == 2 for c in geom):
-        xx, yy = zip(*geom)
-        return list(zip(*project(xx, yy)))  # type: ignore
+    with contextlib.suppress(TypeError):
+        mp = cast("MultiPoint", ops.transform(project, MultiPoint(geom)))
+        return [(p.x, p.y) for p in mp.geoms]  # type: ignore
 
     gtypes = (
         "a list of coordinates such as [(x1, y1), ...],"
@@ -586,6 +634,33 @@ def match_crs(geom: G, in_crs: CRSTYPE, out_crs: CRSTYPE) -> G:
     raise InputTypeError("geom", gtypes)
 
 
+def esri_query(
+    geom: (LineString | Polygon | Point | MultiPoint | tuple[float, float, float, float]),
+    geo_crs: CRSTYPE,
+    out_crs: CRSTYPE,
+) -> Mapping[str, str]:
+    """Generate geometry queries based on ESRI template."""
+    geom = match_crs(geom, geo_crs, out_crs)
+    wkid = cast("int", pyproj.CRS(out_crs).to_epsg())
+
+    if isinstance(geom, tuple) and len(geom) == 4:
+        return ESRIGeomQuery(geom, wkid).bbox()
+
+    if isinstance(geom, Point):
+        return ESRIGeomQuery((geom.x, geom.y), wkid).point()
+
+    if isinstance(geom, MultiPoint):
+        return ESRIGeomQuery([(g.x, g.y) for g in geom.geoms], wkid).multipoint()
+
+    if isinstance(geom, Polygon):
+        return ESRIGeomQuery(geom, wkid).polygon()
+
+    if isinstance(geom, LineString):
+        return ESRIGeomQuery(geom, wkid).polyline()
+
+    raise InputTypeError("geom", "LineString, Polygon, Point, MultiPoint, tuple")
+
+
 def check_bbox(bbox: tuple[float, float, float, float]) -> None:
     """Check if an input inbox is a tuple of length 4."""
     if not (isinstance(bbox, tuple) and len(bbox) == 4):
@@ -686,7 +761,7 @@ def validate_crs(crs: CRSTYPE) -> str:
         Validated CRS as a string.
     """
     try:
-        return pyproj.CRS(crs).to_string()  # type: ignore
+        return pyproj.CRS(crs).to_string()
     except ProjCRSError as ex:
         raise InputTypeError("crs", "a valid CRS") from ex
 
